@@ -19,8 +19,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"math/rand"
-	"net/url"
+	"math/rand/v2"
 	"reflect"
 	"strings"
 	"time"
@@ -42,9 +41,12 @@ func New(deps *deps.Deps) *Namespace {
 	}
 	loc := langs.GetLocation(language)
 
+	dCache := maps.NewCacheWithOptions[dKey, []int](maps.CacheOptions{Size: 100})
+
 	return &Namespace{
 		loc:      loc,
 		sortComp: compare.New(loc, true),
+		dCache:   dCache,
 		deps:     deps,
 	}
 }
@@ -53,6 +55,7 @@ func New(deps *deps.Deps) *Namespace {
 type Namespace struct {
 	loc      *time.Location
 	sortComp *compare.Namespace
+	dCache   *maps.Cache[dKey, []int]
 	deps     *deps.Deps
 }
 
@@ -126,7 +129,7 @@ func (ns *Namespace) Delimit(ctx context.Context, l, sep any, last ...any) (stri
 		lv = reflect.ValueOf(sortSeq)
 		fallthrough
 	case reflect.Array, reflect.Slice, reflect.String:
-		for i := 0; i < lv.Len(); i++ {
+		for i := range lv.Len() {
 			val := lv.Index(i).Interface()
 			valStr, err := cast.ToStringE(val)
 			if err != nil {
@@ -166,7 +169,7 @@ func (ns *Namespace) Dictionary(values ...any) (map[string]any, error) {
 		case string:
 			key = v
 		case []string:
-			for i := 0; i < len(v)-1; i++ {
+			for i := range len(v) - 1 {
 				key = v[i]
 				var m map[string]any
 				v, found := dict[key]
@@ -236,7 +239,7 @@ func (ns *Namespace) In(l any, v any) (bool, error) {
 
 	switch lv.Kind() {
 	case reflect.Array, reflect.Slice:
-		for i := 0; i < lv.Len(); i++ {
+		for i := range lv.Len() {
 			lvv, isNil := indirectInterface(lv.Index(i))
 			if isNil {
 				continue
@@ -278,13 +281,13 @@ func (ns *Namespace) Intersect(l1, l2 any) (any, error) {
 		ins = &intersector{r: reflect.MakeSlice(l1v.Type(), 0, 0), seen: make(map[any]bool)}
 		switch l2v.Kind() {
 		case reflect.Array, reflect.Slice:
-			for i := 0; i < l1v.Len(); i++ {
+			for i := range l1v.Len() {
 				l1vv := l1v.Index(i)
 				if !l1vv.Type().Comparable() {
 					return make([]any, 0), errors.New("intersect does not support slices or arrays of uncomparable types")
 				}
 
-				for j := 0; j < l2v.Len(); j++ {
+				for j := range l2v.Len() {
 					l2vv := l2v.Index(j)
 					if !l2vv.Type().Comparable() {
 						return make([]any, 0), errors.New("intersect does not support slices or arrays of uncomparable types")
@@ -383,47 +386,6 @@ func (ns *Namespace) Last(limit any, l any) (any, error) {
 	return seqv.Slice(seqv.Len()-limitv, seqv.Len()).Interface(), nil
 }
 
-// Querify encodes the given params in URL-encoded form ("bar=baz&foo=quux") sorted by key.
-func (ns *Namespace) Querify(params ...any) (string, error) {
-	qs := url.Values{}
-
-	if len(params) == 1 {
-		switch v := params[0].(type) {
-		case []string:
-			if len(v)%2 != 0 {
-				return "", errors.New("invalid query")
-			}
-
-			for i := 0; i < len(v); i += 2 {
-				qs.Add(v[i], v[i+1])
-			}
-
-			return qs.Encode(), nil
-
-		case []any:
-			params = v
-
-		default:
-			return "", errors.New("query keys must be strings")
-		}
-	}
-
-	if len(params)%2 != 0 {
-		return "", errors.New("invalid query")
-	}
-
-	for i := 0; i < len(params); i += 2 {
-		switch v := params[i].(type) {
-		case string:
-			qs.Add(v, fmt.Sprintf("%v", params[i+1]))
-		default:
-			return "", errors.New("query keys must be strings")
-		}
-	}
-
-	return qs.Encode(), nil
-}
-
 // Reverse creates a copy of the list l and reverses it.
 func (ns *Namespace) Reverse(l any) (any, error) {
 	if l == nil {
@@ -447,6 +409,11 @@ func (ns *Namespace) Reverse(l any) (any, error) {
 	return sliceCopy.Interface(), nil
 }
 
+// Sanity check for slices created by Seq and D.
+const maxSeqSize = 1000000
+
+var errSeqSizeExceedsLimit = errors.New("size of result exceeds limit")
+
 // Seq creates a sequence of integers from args. It's named and used as GNU's seq.
 //
 // Examples:
@@ -500,14 +467,14 @@ func (ns *Namespace) Seq(args ...any) ([]int, error) {
 	}
 
 	// sanity check
-	if last < -100000 {
-		return nil, errors.New("size of result exceeds limit")
+	if last < -maxSeqSize {
+		return nil, errSeqSizeExceedsLimit
 	}
 	size := ((last - first) / inc) + 1
 
 	// sanity check
-	if size <= 0 || size > 2000 {
-		return nil, errors.New("size of result exceeds limit")
+	if size <= 0 || size > maxSeqSize {
+		return nil, errSeqSizeExceedsLimit
 	}
 
 	seq := make([]int, size)
@@ -562,6 +529,35 @@ func (ns *Namespace) Slice(args ...any) any {
 	return collections.Slice(args...)
 }
 
+type dKey struct {
+	seed uint64
+	n    int
+	hi   int
+}
+
+// D returns a slice of n unique random numbers in the range [0, hi) using the provded seed,
+// using  J. S. Vitter's Method D for sequential random sampling, from Vitter, J.S.
+// - An Efficient Algorithm for Sequential Random Sampling - ACM Trans. Math. Software 11 (1985), 37-57.
+// See  https://getkerf.wordpress.com/2016/03/30/the-best-algorithm-no-one-knows-about/
+func (ns *Namespace) D(seed, n, hi int) []int {
+	key := dKey{seed: cast.ToUint64(seed), n: n, hi: hi}
+	if key.n <= 0 || key.hi <= 0 || key.n > key.hi {
+		return nil
+	}
+	if key.n > maxSeqSize {
+		panic(errSeqSizeExceedsLimit)
+	}
+	v, _ := ns.dCache.GetOrCreate(key, func() ([]int, error) {
+		prng := rand.New(rand.NewPCG(key.seed, 0))
+		result := make([]int, 0, key.n)
+		_d(prng, key.n, key.hi, func(i int) {
+			result = append(result, i)
+		})
+		return result, nil
+	})
+	return v
+}
+
 type intersector struct {
 	r    reflect.Value
 	seen map[any]bool
@@ -632,7 +628,7 @@ func (ns *Namespace) Union(l1, l2 any) (any, error) {
 				isNil bool
 			)
 
-			for i := 0; i < l1v.Len(); i++ {
+			for i := range l1v.Len() {
 				l1vv, isNil = indirectInterface(l1v.Index(i))
 
 				if !l1vv.Type().Comparable() {
@@ -652,7 +648,7 @@ func (ns *Namespace) Union(l1, l2 any) (any, error) {
 				}
 			}
 
-			for j := 0; j < l2v.Len(); j++ {
+			for j := range l2v.Len() {
 				l2vv := l2v.Index(j)
 
 				switch kind := l1vv.Kind(); {
@@ -703,7 +699,7 @@ func (ns *Namespace) Uniq(l any) (any, error) {
 
 	seen := make(map[any]bool)
 
-	for i := 0; i < v.Len(); i++ {
+	for i := range v.Len() {
 		ev, _ := indirectInterface(v.Index(i))
 
 		key := normalize(ev)
